
// g++ -O2 -Wall -o boost_rand.x boost_rand.cpp -lstdc++ -lboost_signals

#include <boost/signals/signal0.hpp>

#include "common.h"

typedef boost::signal0<void> Sig0Type ;

struct Slot
{
  void operator() ()
  { }
} ;

typedef std::vector<Sig0Type*> Sig0Array ;
typedef std::vector<Slot*> SlotArray ;
typedef std::vector<boost::signals::connection> ConnArray ;

int main(int argc, char* argv[])
{
  unsigned nb = 100 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 100 ;
    
  Sig0Array signals(nb, 0) ;
  SlotArray slots(nb, 0) ;
  ConnArray conn(1000000L) ;
  unsigned i ;
  for (i = 0 ; i < nb ; ++i)
  {
    signals[i] = new Sig0Type ;
    slots[i] = new Slot ;
  }
  
  std::cout << " " << std::setw(10) << nb ;
  unsigned long current_data_size = data_size() ;

  srandom(1) ;
  unsigned nb_actions = 0 ;
 
  init_chrono() ;
  while ( elapsed() < 2 )
  {
    conn[nb_actions] = signals[random()%nb]->connect(*(slots[random()%nb])) ;
    ++nb_actions ;
  }
  unsigned nb_max_conn = nb_actions ;
  print_speed(nb_actions) ;
  std::cout << std::setw(10) << data_size()-current_data_size ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    (*(signals[i]))() ;
  double e = elapsed() ;
  if ( e < 1.0 )
  {
    unsigned nb_rep = (unsigned)(1.0/e) ;
    init_chrono() ;
    for (unsigned rep = 0 ; rep < nb_rep ; ++rep)
      for (i = 0 ; i < nb ; ++i)
        (*(signals[i]))() ;
    print_speed(nb_rep*nb) ;
  }
  else
    print_speed(e, nb) ;

  current_data_size = data_size() ;
  init_chrono() ;
  nb_actions = 0 ;
  while ( elapsed() < 2 )
  {
    conn[random()%nb_max_conn].disconnect() ;
    ++nb_actions ;
  }
  print_speed(nb_actions) ;
  std::cout << std::setw(10) << current_data_size-data_size() << "\n" ;

  return 0 ;
}
