
// g++ -O2 -Wall -o libsig_m2o.x libsig_m2o.cpp -I$HOME/opt/include/sigc++-2.0 -I/home/yves/opt/lib/sigc++-2.0/include/ -L$HOME/opt/lib -lstdc++ -lsigc-2.0

#include "common.h"

#include <sigc++/sigc++.h>

typedef sigc::signal0<void> Sig0Type ;

struct Slot : public sigc::trackable
{
  Slot() : sigc::trackable() { }
  void f() { }
} ;

typedef std::vector<Sig0Type*> Sig0Array ;
typedef std::vector<Slot*> SlotArray ;
typedef std::vector<Sig0Type::iterator> ConnArray ;

int main(int argc, char* argv[])
{
  unsigned nb = 100 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 100 ;

  Sig0Array signals(nb, 0) ;
  unsigned i ;
  for (i = 0 ; i < nb ; ++i)
    signals[i] = new Sig0Type ;

  Slot slot ;
  ConnArray conn(nb) ;
  std::cout << " " << std::setw(10) << nb ;

  unsigned long current_data_size = data_size() ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    conn[i] = signals[i]->connect(sigc::mem_fun(&slot, &Slot::f)) ;
  print_speed(nb) ;
  std::cout << std::setw(10) << data_size()-current_data_size ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    signals[i]->emit() ;
  double e = elapsed() ;
  if ( e < 1.0 )
  {
    unsigned nb_rep = (unsigned)(1.0/e) ;
    init_chrono() ;
    for (unsigned rep = 0 ; rep < nb_rep ; ++rep)
      for (i = 0 ; i < nb ; ++i)
        signals[i]->emit() ;
    print_speed(nb_rep*nb) ;
  }
  else
    print_speed(e, nb) ;

  current_data_size = data_size() ;
  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    conn[i]->disconnect() ;
  print_speed(nb) ;
  std::cout << std::setw(10) << current_data_size-data_size() << "\n" ;

  return 0 ;
}
