
// g++ -O2 -Wall -o sigslot_m2o.x sigslot_m2o.cpp -lstdc++

#include "common.h"

#define SIGSLOT_PURE_ISO
#include "sigslot.h"

typedef sigslot::signal0<> Sig0Type ;

struct Slot : public sigslot::has_slots<>
{
  Slot() : sigslot::has_slots<>() { }
  void f() { }
} ;

typedef std::vector<Sig0Type*> Sig0Array ;
typedef std::vector<Slot*> SlotArray ;

int main(int argc, char* argv[])
{
  unsigned nb = 100 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 100 ;

  Sig0Array signals(nb, 0) ;
  unsigned i ;
  for (i = 0 ; i < nb ; ++i)
    signals[i] = new Sig0Type ;

  Slot slot ;
  std::cout << " " << std::setw(10) << nb ;

  unsigned long current_data_size = data_size() ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    signals[i]->connect(&slot, &Slot::f) ;
  print_speed(nb) ;
  std::cout << std::setw(10) << data_size()-current_data_size ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    signals[i]->emit() ;
  double e = elapsed() ;
  if ( e < 1.0 )
  {
    unsigned nb_rep = (unsigned)(1.0/e) ;
    init_chrono() ;
    for (unsigned rep = 0 ; rep < nb_rep ; ++rep)
      for (i = 0 ; i < nb ; ++i)
        signals[i]->emit() ;
    print_speed(nb_rep*nb) ;
  }
  else
    print_speed(e, nb) ;

  current_data_size = data_size() ;
  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    signals[i]->disconnect(&slot) ;
  print_speed(nb) ;
  std::cout << std::setw(10) << current_data_size-data_size() << "\n" ;

  return 0 ;
}
