
// g++ -O2 -Wall -o slotsig_rand.x ../slotsig/slotsig_bases.cpp slotsig_rand.cpp -lstdc++

#include "common.h"

#include "../slotsig/slotsig_0.h"

typedef SlotSig::Sig0<void> Sig0Type ;

struct Slot : public SlotSig::SlotsSetBase
{
  Slot() : SlotSig::SlotsSetBase() { }
  void f() { }
} ;

typedef std::vector<Sig0Type*> Sig0Array ;
typedef std::vector<Slot*> SlotArray ;

int main(int argc, char* argv[])
{
  unsigned nb = 100 ;
  if ( argc > 1 )
    nb = atol(argv[1]) ;
  if ( nb == 0 )
    nb = 100 ;
    
  Sig0Array signals(nb, 0) ;
  SlotArray slots(nb, 0) ;
  unsigned i ;
  for (i = 0 ; i < nb ; ++i)
  {
    signals[i] = new Sig0Type ;
    slots[i] = new Slot ;
  }
  
  std::cout << " " << std::setw(10) << nb ;
  unsigned long current_data_size = data_size() ;

  srandom(1) ;
  unsigned nb_actions = 0 ;
 
  init_chrono() ;
  while ( elapsed() < 2 )
  {
    signals[random()%nb]->connect(slots[random()%nb], &Slot::f) ;
    ++nb_actions ;
  }
  print_speed(nb_actions) ;
  std::cout << std::setw(10) << data_size()-current_data_size ;

  init_chrono() ;
  for (i = 0 ; i < nb ; ++i)
    signals[i]->run() ;
  double e = elapsed() ;
  if ( e < 1.0 )
  {
    unsigned nb_rep = (unsigned)(1.0/e) ;
    init_chrono() ;
    for (unsigned rep = 0 ; rep < nb_rep ; ++rep)
      for (i = 0 ; i < nb ; ++i)
        signals[i]->run() ;
    print_speed(nb_rep*nb) ;
  }
  else
    print_speed(e, nb) ;

  current_data_size = data_size() ;
  init_chrono() ;
  nb_actions = 0 ;
  while ( elapsed() < 2 )
  {
    signals[random()%nb]->disconnect(slots[random()%nb], &Slot::f) ;
    ++nb_actions ;
  }
  print_speed(nb_actions) ;
  std::cout << std::setw(10) << current_data_size-data_size() << "\n" ;

  return 0 ;
}
