
// Compile command :
// g++ -Wall -o test_3.x -I.. -lstdc++ ../slotsig/slotsig_bases.cpp test_3.cpp
#include <iostream>
#include <string>
#include <slotsig/slotsig_3.h>

class C1 : public SlotSig::SlotsSetBase
{
  public:
    C1() : SlotSig::SlotsSetBase() { }

    std::string m1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::m1 (" << s << ")\n" ;
      return "C1::m1" ;
    }

    std::string m2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::m2 (" << s << ")\n" ;
      return "C1::m2" ;
    }

    virtual std::string vm1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::vm1 (" << s << ")\n" ;
      return "C1::vm1" ;
    }

    virtual std::string vm2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C1::vm2 (" << s << ")\n" ;
      return "C1::vm2" ;
    }
} ; // C1

class C2 : public C1
{
  public:
    C2() : C1() { }

    virtual std::string vm1(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C2::vm1 (" << s << ")\n" ;
      return "C2::vm1" ;
    }

    virtual std::string vm2(const std::string& s0, const std::string& s1, const std::string& s2) const
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C2::vm2 (" << s << ")\n" ;
      return "C2::vm2" ;
    }
} ; // C2

class C3
{
  public:
    C3() { }
    virtual std::string m(const std::string& s0, const std::string& s1, const std::string& s2)
    {
      std::string s = s0 + ", " + s1 + ", " + s2 ;
      std::cout << "C3::m (" << s << ")\n" ;
      return "C3::m" ;
    }
} ; // C3

std::string global_slot(const std::string& s0, const std::string& s1, const std::string& s2)
{
  std::string s = s0 + ", " + s1 + ", " + s2 ;
  std::cout << "::global_slot (" << s << ")\n" ;
  return "::global_slot" ;
}

struct Accum
{
  Accum() : str("") { }

  void operator() (const std::string& s)
  { str += (s + " ") ; }

  std::string str ;
} ; // Accum

struct Pred
{
  Pred() : cpt(0) { }

  bool operator() (const std::string&)
  { return ( ((++cpt) % 3) != 0 ) ; }

  unsigned cpt ;
} ; // Pred

int main()
{
  std::cout << "--- Test program START ---\n" ;

  std::cout << "---\n"
    << "--- Basic tests\n"
    << "---\n\n" ;
  C1* c1 = new C1 ;
  C2* c2 = new C2 ;
  typedef SlotSig::Sig3<std::string, const std::string&, const std::string&, const std::string&> SigType ;
  SigType* sig1 = new SigType(false) ;
  SigType* sig2 = new SigType ;
  Accum accum ;
  Pred pred ;

  std::cout << "--- Connecting all methods from c1 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e1_0", "e1_1", "e1_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #2 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e2_0", "e2_1", "e2_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #3 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e3_0", "e3_1", "e3_2")) << "\n" ;

  std::cout << "--- Re-connecting all methods from c1 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #4...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e4_0", "e4_1", "e4_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #5 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e5_0", "e5_1", "e5_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #6 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e6_0", "e6_1", "e6_2")) << "\n" ;

  std::cout << "--- Ensuring uniqueness of slots in sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->set_ensure_unique(true)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->enforce_unique()) << "\n" ;
  std::cout << "--- Emitting sig1 #7...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e7_0", "e7_1", "e7_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #8 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e8_0", "e8_1", "e8_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #9 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e9_0", "e9_1", "e9_2")) << "\n" ;

  std::cout << "--- Connecting all methods from c2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #10...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e10_0", "e10_1", "e10_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #11 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e11_0", "e11_1", "e11_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #12 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e12_0", "e12_1", "e12_2")) << "\n" ;

  std::cout << "--- Re-connecting all methods from c2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c2, &C2::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #13...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e13_0", "e13_1", "e13_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #14 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e14_0", "e14_1", "e14_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #15 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e15_0", "e15_1", "e15_2")) << "\n" ;

  std::cout << "--- Connecting all methods from c1 and c2 to sig2...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::vm2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c2, &C2::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c2, &C2::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c2, &C2::vm1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c2, &C2::vm2)) << "\n" ;
  std::cout << "--- Emitting sig2 #16...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e16_0", "e16_1", "e16_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #17 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig2->run(accum, "e17_0", "e17_1", "e17_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #18 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run_if(pred, "e18_0", "e18_1", "e18_2")) << "\n" ;

  std::cout << "--- Disconnecting global_slot, c1->m1, c1->vm2, c2->m2 and c2->vm1 from sig2...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->disconnect(global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->disconnect(c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->disconnect(c1, &C1::vm2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->disconnect(c2, &C2::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->disconnect(c2, &C2::vm1)) << "\n" ;
  std::cout << "--- Emitting sig1 #19...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e19_0", "e19_1", "e19_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #20 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e20_0", "e20_1", "e20_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #21 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e21_0", "e21_1", "e21_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #22...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e22_0", "e22_1", "e22_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #23 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig2->run(accum, "e23_0", "e23_1", "e23_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #24 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run_if(pred, "e24_0", "e24_1", "e24_2")) << "\n" ;

  std::cout << "--- Deleting c1...\n" ;
  delete c1 ;
  c1 = 0 ;
  std::cout << "--- Emitting sig1 #25...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e25_0", "e25_1", "e25_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #26 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e26_0", "e26_1", "e26_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #27 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e27_0", "e27_1", "e27_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #28...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e28_0", "e28_1", "e28_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #29 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig2->run(accum, "e29_0", "e29_1", "e29_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #30 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run_if(pred, "e30_0", "e30_1", "e30_2")) << "\n" ;

  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  sig1 = 0 ;
  std::cout << "--- Emitting sig2 #31...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e31_0", "e31_1", "e31_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #32 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig2->run(accum, "e32_0", "e32_1", "e32_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #33 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run_if(pred, "e33_0", "e33_1", "e33_2")) << "\n" ;

  std::cout << "--- Deleting c2...\n" ;
  delete c2 ;
  std::cout << "--- Emitting sig2 #34...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e34_0", "e34_1", "e34_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #35 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig2->run(accum, "e35_0", "e35_1", "e35_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig2 #36 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run_if(pred, "e36_0", "e36_1", "e36_2")) << "\n" ;

  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;
  
  std::cout << "\n---\n"
    << "--- Chaining tests\n"
    << "---\n\n" ;
    
  c1 = new C1() ;
  sig1 = new SigType() ;
  sig2 = new SigType() ;
  
  std::cout << "--- Connecting global_slot, C1::m1 and C1::vm2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #37...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e37_0", "e37_1", "e37_2")) << "\n" ;
  
  std::cout << "--- Connecting C1::m2 and C1::vm1 to sig2...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::vm1)) << "\n" ;
  std::cout << "--- Emitting sig2 #38...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e38_0", "e38_1", "e38_2")) << "\n" ;
 
  std::cout << "--- Connecting sig2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect(sig2)) << "\n" ;
  std::cout << "--- Emitting sig1 #39...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e39_0", "e39_1", "e39_2")) << "\n" ;
  std::cout << "--- Emitting sig1 #40 with accumulator...\n" ;
  accum.str = "" ;
  std::cout << "  " << SlotSig::error2str(sig1->run(accum, "e40_0", "e40_1", "e40_2")) << "\n" ;
  std::cout << "accum.str = " << accum.str << "\n" ;
  std::cout << "--- Emitting sig1 #41 with predicate (run_if)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run_if(pred, "e41_0", "e41_1", "e41_2")) << "\n" ;

  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;
  std::cout << "--- Emitting sig1 #42...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e42_0", "e42_1", "e42_2")) << "\n" ;

  std::cout << "--- Recreating sig2 and reconnecting C1::m2 and C1::vm1 to sig2...\n" ;
  sig2 = new SigType() ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::m2)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect (c1, &C1::vm1)) << "\n" ;
  std::cout << "--- Reconnecting sig2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect(sig2)) << "\n" ;
  std::cout << "--- Emitting sig1 #43...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e43_0", "e43_1", "e43_2")) << "\n" ;
  std::cout << "--- Connecting sig1 to sig2 (creating cycle)...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->connect(sig1)) << "\n" ;
  std::cout << "--- Emitting sig1 #44...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e44_0", "e44_1", "e44_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #45...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e45_0", "e45_1", "e45_2")) << "\n" ;
  
  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  std::cout << "--- Emitting sig2 #46...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e46_0", "e46_1", "e46_2")) << "\n" ;
  
  std::cout << "--- Deleting c1...\n" ;
  delete c1 ;
  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;
  
  std::cout << "\n---\n"
    << "--- Cloning tests\n"
    << "---\n\n" ;
  
  c1 = new C1 ;
  sig1 = new SigType() ;
  
  std::cout << "--- Connecting global_slot, C1::m1 and C1::vm2 to sig1...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (global_slot)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::m1)) << "\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->connect (c1, &C1::vm2)) << "\n" ;
  std::cout << "--- Emitting sig1 #47...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e47_0", "e47_1", "e47_2")) << "\n" ;
  
  std::cout << "--- Cloning sig1 into sig2...\n" ;
  sig2 = sig1->clone() ;
  std::cout << "--- Emitting sig2 #48...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e48_0", "e48_1", "e48_2")) << "\n" ;
  
  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  std::cout << "--- Emitting sig2 #49...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e49_0", "e49_1", "e49_2")) << "\n" ;

  std::cout << "--- Creating sig1 as clone of sig2...\n" ;
  sig1 = new SigType(*sig2) ;  
  std::cout << "--- Emitting sig1 #50...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e50_0", "e50_1", "e50_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #51...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e51_0", "e51_1", "e51_2")) << "\n" ;

  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;
  std::cout << "--- Emitting sig1 #52...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e52_0", "e52_1", "e52_2")) << "\n" ;
  
  std::cout << "--- Creating sig2, connecting C1::m2...\n" ;
  sig2 = new SigType ;
  std::cout << "   " << SlotSig::error2str(sig2->connect(c1, &C1::m2)) << "\n" ;
  std::cout << "--- Emitting sig1 #53...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e53_0", "e53_1", "e53_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #54...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e54_0", "e54_1", "e54_2")) << "\n" ;
  
  std::cout << "--- Copying sig1 into sig2...\n" ;
  *sig2 = *sig1 ;
  std::cout << "--- Emitting sig1 #55...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e55_0", "e55_1", "e55_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #56...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e56_0", "e56_1", "e56_2")) << "\n" ;
  
  std::cout << "--- Creating sig3, connecting C1::m2...\n" ;
  SigType* sig3 = new SigType() ;
  std::cout << "   " << SlotSig::error2str(sig3->connect(c1, &C1::m2)) << "\n" ;
  std::cout << "--- Emitting sig3 #57...\n" ;
  std::cout << "  " << SlotSig::error2str(sig3->run("e57_0", "e57_1", "e57_2")) << "\n" ;
  std::cout << "--- Connecting sig3 to sig1...\n" ;
  std::cout << "   " << SlotSig::error2str(sig1->connect(sig3)) << "\n" ;
  std::cout << "--- Copying sig1 into sig2...\n" ;
  *sig2 = *sig1 ;
  std::cout << "--- Emitting sig1 #58...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e58_0", "e58_1", "e58_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #59...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e59_0", "e59_1", "e59_2")) << "\n" ;
  
  std::cout << "--- Deleting sig3...\n" ;
  delete sig3 ;
  std::cout << "--- Emitting sig1 #60...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e60_0", "e60_1", "e60_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #61...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e61_0", "e61_1", "e61_2")) << "\n" ;

  std::cout << "--- Deleting c1...\n" ;
  delete c1 ;
  c1 = 0 ;
  std::cout << "--- Emitting sig1 #62...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e62_0", "e62_1", "e62_2")) << "\n" ;
  std::cout << "--- Emitting sig2 #63...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e63_0", "e63_1", "e63_2")) << "\n" ;

  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  std::cout << "--- Emitting sig2 #64...\n" ;
  std::cout << "  " << SlotSig::error2str(sig2->run("e64_0", "e64_1", "e64_2")) << "\n" ;
  
  std::cout << "--- Deleting sig2...\n" ;
  delete sig2 ;
  
  std::cout << "\n---\n"
    << "--- \"Free\" connection tests\n"
    << "---\n\n" ;
    
  C3* c3 = new C3() ;
  sig1 = new SigType() ;
  
  std::cout << "--- Connecting C3::m to sig1...\n" ;
  std::cout << "   " << SlotSig::error2str(sig1->connect(c3, &C3::m)) << "\n" ;
  std::cout << "--- Emitting sig1 #65...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e65_0", "e65_1", "e65_2")) << "\n" ;

  std::cout << "--- Disconnecting C3::m to sig1...\n" ;
  std::cout << "   " << SlotSig::error2str(sig1->disconnect(c3, &C3::m)) << "\n" ;
  std::cout << "--- Emitting sig1 #66...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e66_0", "e66_1", "e66_2")) << "\n" ;
  
  std::cout << "--- Re-connecting C3::m to sig1...\n" ;
  std::cout << "   " << SlotSig::error2str(sig1->connect(c3, &C3::m)) << "\n" ;
  std::cout << "--- Emitting sig1 #67...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e67_0", "e67_1", "e67_2")) << "\n" ;
  
  std::cout << "--- Announcing c3 deletion...\n" ;
  SlotSig::about_to_delete(c3) ;
  std::cout << "--- Emitting sig1 #68...\n" ;
  std::cout << "  " << SlotSig::error2str(sig1->run("e68_0", "e68_1", "e68_2")) << "\n" ;
  
  std::cout << "--- Deleting sig1...\n" ;
  delete sig1 ;
  
  std::cout << "\n--- Test program END ----\n" ;
  return 0 ;
}
